resource "aws_cloudwatch_log_group" "lambda" {
  name              = "/aws/lambda/${var.function_name}"
  retention_in_days = var.cloudwatch_retention
  tags              = var.tags
}

resource "aws_lambda_function" "lambda" {
  function_name                  = var.function_name
  filename                       = var.filename
  package_type                   = var.package_type
  source_code_hash               = filebase64sha256(var.filename)
  role                           = var.lambda_role_arn
  handler                        = var.handler
  memory_size                    = var.memory_size
  description                    = var.description
  timeout                        = var.timeout
  runtime                        = var.runtime
  publish                        = var.publish
  reserved_concurrent_executions = var.reserved_concurrent_executions
  tags                           = var.tags
  layers                         = var.layers

  vpc_config {
    subnet_ids         = var.subnet_ids
    security_group_ids = var.security_group_ids
  }

  dynamic "dead_letter_config" {
    for_each = length(var.dlq_target_arn) > 0 ? ["true"] : []
    content {
      target_arn = var.dlq_target_arn
    }
  }

  environment {
    variables = merge(var.environment, var.extra_environment, var.infra_mount, var.default_env_vars)
  }
}

# if asyncInv is enabled
resource "aws_lambda_function_event_invoke_config" "lambda" {
  count                        = var.asyncInv_enable ? 1 : 0
  function_name                = aws_lambda_function.lambda.function_name
  maximum_event_age_in_seconds = var.max_event_age
  maximum_retry_attempts       = var.max_retry
}

# If resource-based policy is enabled/notempty
resource "aws_lambda_permission" "resource_based_policy_lambda_aws_service" {
  for_each       = { for rp in var.resource_based_policies_lambda_aws_service : rp.statement_id => rp }
  statement_id   = lookup(each.value, "statement_id", null)
  action         = lookup(each.value, "action", null)
  function_name  = aws_lambda_function.lambda.function_name
  principal      = lookup(each.value, "principal", null)
  source_arn     = lookup(each.value, "source_arn", null)
  source_account = lookup(each.value, "source_account", null)
}

##
## Event Mappings
##

# Kinesis
resource "aws_lambda_event_source_mapping" "kinesis_stream_source" {
  count             = var.kinesis_enable ? 1 : 0
  batch_size        = var.batch_size
  enabled           = var.event_source_mapping_enabled
  event_source_arn  = var.event_source_arn
  function_name     = var.function_name
  starting_position = var.starting_position
  depends_on        = [aws_lambda_function.lambda]
}

# Dynamodb
resource "aws_lambda_event_source_mapping" "dynamodb_stream_source" {
  count             = var.dynamodb_enable ? 1 : 0
  batch_size        = var.batch_size
  enabled           = var.event_source_mapping_enabled
  event_source_arn  = var.event_source_arn
  function_name     = var.function_name
  starting_position = var.starting_position
  depends_on        = [aws_lambda_function.lambda]
}

# Sqs
resource "aws_lambda_event_source_mapping" "sqs_stream_source" {
  count            = var.sqs_enable ? 1 : 0
  batch_size       = var.batch_size
  enabled          = var.event_source_mapping_enabled
  event_source_arn = var.event_source_arn
  function_name    = var.function_name
  depends_on       = [aws_lambda_function.lambda]
}

resource "aws_iam_policy" "invoke-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-lambda-invoke"
  path        = "/"
  description = "Lambda invocation policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "lambda:InvokeFunction"
      ],
      "Effect": "Allow",
      "Resource": "${aws_lambda_function.lambda.arn}"
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }
  tags = var.tags
}
